@@ -38,7 +38,10 @@ def get_dynamo_connection(region_name=None, endpoint_url=None, force_live=False,
             params["endpoint_url"] = endpoint_url
     if not force_live:
         if force_local or "pytest" in sys.modules:
-            params["endpoint_url"] = "http://localhost:8989"
+            usefule_endpoint = os.environ.get("DYNAMODB_LOCAL_URL", endpoint_url)
+            if not usefule_endpoint:
+                raise Exception("in local and test, endpoint_url is required or DYNAMODB_LOCAL_URL environment with local url")
+            params["endpoint_url"] = usefule_endpoint
             params["region_name"] = "us-west-2"
     return boto3.resource("dynamodb", **params)
     
@@ -54,6 +57,7 @@ class DynamoController(object):
     TYPE_UPDATE = "update"
     TYPE_DELETE = "delete"
     TYPE_FILTER = "filter"
+    TYPE_DELETE_BATCH = "delete_batch"
 
     ORDER_ASC = "asc"
     ORDER_DESC = "desc"
@@ -66,9 +70,14 @@ class DynamoController(object):
         self.table = self.dynamodb.Table(table_name)
         self.debug = debug if debug else os.getenv("LOCAL_USER", False)
 
-    def get(self, primary_keys):
-        """ return a the first element corresponding to the filter_data param"""
-        return self._execute_operation(type=self.TYPE_GET, primary_keys=primary_keys)
+    def get(self, primary_keys, columns_to_return=None):
+        """return a the first element corresponding to the filter_data param
+
+        Args:
+            primary_keys (dict): Structure with PK definition
+            columns_to_return (string, optional): List of columns to be returned
+        """
+        return self._execute_operation(type=self.TYPE_GET, primary_keys=primary_keys, columns_to_return=columns_to_return)
 
     def create(self, data):
         """ Creates a new instance in the DB."""
@@ -90,6 +99,13 @@ class DynamoController(object):
             return res_del
         return res
 
+    def delete_batch(self, primary_keys):
+        """ Deletes a list of objects"""
+        res_del = self._execute_operation(type=self.TYPE_DELETE_BATCH, primary_keys=primary_keys)
+        if res_failed(res_del):
+            return res_del
+        return "Deleted item(s): {}".format(primary_keys)
+
 
     def get_filter_fe(self, fe_dict):
         """ generate filter_expression based on fe_dict received in event """
@@ -177,12 +193,7 @@ class DynamoController(object):
                         total_condition = total_condition & new_fe
         return total_condition
 
-
-
-
-
-
-    def filter(self, key, value, order_by=None, limit=50, next=None, index=None, order_type=ORDER_DESC, filter_expression=None, invert_result=False, condition_expression=None):
+    def filter(self, key, value, order_by=None, order_by_value=None, limit=500, next=None, index=None, order_type=ORDER_DESC, filter_expression=None, invert_result=False, condition_expression=None, projection_expression=None, expression_attribute_names=None, columns_to_return=None):
         """ Return all orders related to this user
             Params:
                 - filter_expression is of type: FilterExpression (boto3), Veryyy flexible
@@ -191,17 +202,21 @@ class DynamoController(object):
                     - Exception: if no request is provided
                     - ValueError: is filter_expression has a value
         """
+        index_name = None
+        limit = int(limit) #forcing limit to be int
         if index:
             index_name = index
-        elif key and order_by:
-            index_name = "{}-{}-index".format(key, order_by)
-        else:
-            index_name = "{}-index".format(key)
+        elif key:
+            if order_by:
+                index_name = "{}-{}-index".format(key, order_by)
+            else:
+                index_name = "{}-index".format(key)
 
         params = {
-            "IndexName": index_name,
             "Limit":limit,
         }
+        if index_name and index_name != 'No':
+            params["IndexName"] = index_name
         if next:
             params["ExclusiveStartKey"] = next
 
@@ -211,9 +226,9 @@ class DynamoController(object):
             condition_expression = json_float_to_decimal(condition_expression)
             condition_expression = self.get_condition_expression_fe(condition_expression)
             if condition_expression:
-                params["KeyConditionExpression"] = Key(key).eq(value) & condition_expression
+                params["KeyConditionExpression"] = Key(key).eq(value) & Key(order_by).begins_with(order_by_value) & condition_expression if order_by and order_by_value else Key(key).eq(value) & condition_expression
         if "KeyConditionExpression" not in params:
-            params["KeyConditionExpression"] = Key(key).eq(value)
+            params["KeyConditionExpression"] = Key(key).eq(value) & Key(order_by).begins_with(order_by_value) if order_by and order_by_value else Key(key).eq(value)
 
 
         #if request:
@@ -227,10 +242,18 @@ class DynamoController(object):
 
         if filter_expression:
             params["FilterExpression"] = filter_expression
+            
+        if projection_expression:
+            params["ProjectionExpression"] = projection_expression
+            
+        if expression_attribute_names:
+            params["ExpressionAttributeNames"] = expression_attribute_names
 
         #order_type = order if order else order_type
         if order_type == self.ORDER_DESC:
             params["ScanIndexForward"] = False
+
+        params["columns_to_return"] = columns_to_return
         
         # pulling data from DB
         data, count, next = self._execute_operation(type=self.TYPE_FILTER, primary_keys=params)
@@ -269,11 +292,11 @@ class DynamoController(object):
         return filter_expression,order
     '''
     
-    def _execute_operation(self, type, primary_keys=None, data=None):
+    def _execute_operation(self, type, primary_keys=None, data=None, columns_to_return=None):
         """ Execute the dynamo operation and return a proper response (success or error)"""
         try:
             if type == self.TYPE_GET:
-                d_res = self.table.get_item(Key=primary_keys)
+                d_res = self.table.get_item(Key=primary_keys, ProjectionExpression=columns_to_return)
                 if "Item" in d_res:
                     data = d_res["Item"]
                 else:
@@ -295,7 +318,16 @@ class DynamoController(object):
             elif type == self.TYPE_DELETE:
                 d_res = self.table.delete_item(Key=primary_keys)
                 return primary_keys
+            elif type == self.TYPE_DELETE_BATCH:
+                if isinstance(primary_keys, dict):
+                    primary_keys = [primary_keys]
+                with self.table.batch_writer()as batch:
+                    for item in primary_keys:
+                        batch.delete_item(Key=item)
+                return primary_keys
             elif type == self.TYPE_FILTER:
+                if "columns_to_return" not in primary_keys:
+                    primary_keys["ProjectionExpression"] = columns_to_return
                 response = self.table.query(**primary_keys)
                 data = response["Items"] if response["Count"] != 0 else []
                 code = response['ResponseMetadata']['HTTPStatusCode']
